#!/usr/bin/env python
# -*- coding: utf-8 -*-
version = '0.001'
"""

Copyright John Thornton 2012
"""

try:
  import pygtk
  pygtk.require('2.0')
except:
  pass
try:
  import gtk
  import gtk.glade
except:
  print('GTK not available')
  sys.exit(1)

import sqlite3 as lite
import populate

class Buglump:

  def __init__(self):
    self.con = lite.connect('sfc.sqlite')
    self.gladefile = "sfc.glade"
    self.builder = gtk.Builder()
    self.builder.add_from_file(self.gladefile)
    self.builder.connect_signals(self)
    self.populate = populate.populate()

    self.system = "I"

    self.typelist = gtk.ListStore(str, str)
    self.typelist.append(["","Select a Form"])
    self.typelist.append(["I","UNC"])
    self.typelist.append(["I","UNF"])
    self.typelist.append(["I","UNEF"])
    self.typelist.append(["M","ISO"])
    self.typecombo = self.builder.get_object("typecombo")
    self.typecombo.set_model(self.typelist)
    self.typecell = gtk.CellRendererText()
    self.typecombo.pack_start(self.typecell)
    self.typecombo.add_attribute(self.typecell, 'text', 1)
    self.typecombo.set_active(0)

    self.threadlist = gtk.ListStore(str)
    self.tapcombo = self.builder.get_object("tapcombo")
    self.tapcombo.set_model(self.threadlist)
    self.tapcell = gtk.CellRendererText()
    self.tapcombo.pack_start(self.tapcell)
    self.tapcombo.add_attribute(self.tapcell, 'text', 0)

    self.aboutdialog = self.builder.get_object("aboutdialog")
    self.aboutdialog.set_version(version)

    self.window = self.builder.get_object("window1")
    self.window.set_title("Machinist Calculator")
    self.window.show_all()

  def on_typecombo_changed(self, widget, data=None):
    self.index = widget.get_active()
    if self.index > 0:
      self.model = widget.get_model()
      self.item = [self.model[self.index][1]]
      self.system = self.model[self.index][0]
      with self.con:
        self.con.row_factory = lite.Row
        self.cur = self.con.cursor()
        self.cur.execute("SELECT DISTINCT size FROM threads WHERE form = ? ORDER BY major_dia ASC", self.item)
        itemlist = self.cur.fetchall()
      self.threadlist.clear()
      for i in itemlist:
        self.threadlist.append([i["size"]])

  def on_tapcombo_changed(self, widget, data=None):
    self.index = widget.get_active()
    if self.index > 0:
      self.fractional_bits = self.builder.get_object("fractional_cb")
      self.number_bits = self.builder.get_object("number_cb")
      self.letter_bits = self.builder.get_object("letter_cb")
      self.metric_bits = self.builder.get_object("metric_cb")
      self.range = []
      if self.fractional_bits.get_active():
        self.range.append("F")
      if self.number_bits.get_active():
        self.range.append("N")
      if self.letter_bits.get_active():
        self.range.append("L")
      if self.metric_bits.get_active():
        self.range.append("M")
      self.query = 'SELECT * FROM drills WHERE type IN (%s)' % ','.join('?' for i in self.range)
      self.query += " AND diam >= ? AND diam <= ?"
      self.model = widget.get_model()
      self.item = self.model[self.index][0]
      with self.con:
        self.con.row_factory = lite.Row
        self.cur = self.con.cursor()
        self.cur.execute("SELECT * FROM threads WHERE size = '%s'" % self.item)
        itemlist = self.cur.fetchall()
      if self.system == "I":
        self.major_dia = itemlist[0]["major_dia"]
        self.minor_dia = itemlist[0]["minor_dia"]
        self.tpi = itemlist[0]["pitch"]
        self.wire_dia = itemlist[0]["wire_dia"]
        self.pitch_dia = itemlist[0]["pitch_dia"]
      elif self.system == "M":
        self.major_dia = itemlist[0]["major_dia"]*.03937
        self.minor_dia = itemlist[0]["minor_dia"]*.03937
        self.tpi = itemlist[0]["pitch"]*.03937
        self.wire_dia = itemlist[0]["wire_dia"]*.03937
        self.pitch_dia = itemlist[0]["pitch_dia"]*.03937
      if self.major_dia > 0:
        self.builder.get_object("major_diameter").set_text("Major Diameter " + str(self.major_dia) + '"')
      else:
        self.builder.get_object("major_diameter").set_text('n/a')
      if self.minor_dia > 0:
        self.builder.get_object("minor_diameter").set_text("Minor Diameter " + str(self.minor_dia) + '"')
      else:
        self.builder.get_object("minor_diameter").set_text('n/a')
      if self.wire_dia > 0:
        self.builder.get_object("thread_wire").set_text("Thread Wire Diameter " + str(self.wire_dia) + '"')
      else:
        self.builder.get_object("thread_wire").set_text('n/a')
      self.const = itemlist[0]["const"]
      if self.const > 0:
        self.builder.get_object("thread_const").set_text("Thread Constant " + str(self.const) + '"')
      else:
        self.builder.get_object("thread_wire").set_text('n/a')
      if self.pitch_dia > 0:
        self.wire_measure = str(round(self.major_dia - (1.51555/self.tpi)+ (3*self.wire_dia),5))
        self.builder.get_object("over_wire").set_text("Over Wires " + self.wire_measure + '"')
      else:
        self.builder.get_object("thread_wire").set_text('n/a')
      if self.pitch_dia > 0:
        self.builder.get_object("pitch_dia").set_text("Basic Pitch Diameter " + str(self.pitch_dia) + '"')
      else:
        self.builder.get_object("pitch_dia").set_text('n/a')

      self.drill50 = str(round(self.major_dia - ((0.01299 * 50) / self.tpi),5))
      self.builder.get_object("label15").set_text("50% = " + self.drill50 + " Diameter")
      self.drill100 = str(round(self.major_dia - ((0.01299 * 100) / self.tpi),5))
      self.builder.get_object("label17").set_text("100% = " + self.drill100 + " Diameter")
      self.range.append(self.drill100)
      self.range.append(self.drill50)
      with self.con:
        self.con.row_factory = lite.Row
        self.cur = self.con.cursor()
        self.cur.execute(self.query,self.range)
        dlist = self.cur.fetchall()
      self.drilllist = 'Type\n'
      for i in dlist:
        self.drilllist += str(i["type"]) + '\n'
      self.builder.get_object("drill_type").set_text(self.drilllist)
      self.drilllist = 'Size\n'
      for i in dlist:
        self.drilllist += str(i["size"]) + '\n'
      self.builder.get_object("drill_size").set_text(self.drilllist)
      self.drilllist = 'Dia\n'
      for i in dlist:
        self.drilllist += str(i["diam"]) + '\n'
      self.builder.get_object("drill_diameter").set_text(self.drilllist)
      self.drilllist = 'Percent\n'
      for i in dlist:
        self.percent = str(round(((self.major_dia - i["diam"])/0.1299) * self.tpi ,2)*10)
        self.drilllist += self.percent + '%\n'
      self.builder.get_object("thread_percent").set_text(self.drilllist)

  def on_calc_thread_pitch_clicked(self, object, data=None):
    with self.con:
      self.con.row_factory = lite.Row
      self.cur = self.con.cursor()
      self.cur.execute("SELECT * FROM threads WHERE size = '%s'" % self.item)
      itemlist = self.cur.fetchall()
    self.tpi = itemlist[0]["pitch"]
    self.wire_dia = itemlist[0]["wire_size"]

    self.aow = self.builder.get_object("actual_over_wire")
    self.mom = float(self.aow.get_text())
    self.apd = str(round(self.mom-((3*self.wire_dia) - (0.86603 / self.tpi)),5))
    self.builder.get_object("thread_pitch").set_text("Pitch Diameter " + self.apd + '"')

  def on_machine_save_clicked(self, object, data=None):
    print "saved"

  def on_window1_destroy(self, object, data=None):
    print "quit with cancel"
    gtk.main_quit()

  def on_menu_quit_activate(self, menuitem, data=None):
    print "quit from menu"
    gtk.main_quit()

  def on_gtk_about_activate(self, menuitem, data=None):
    self.response = self.aboutdialog.run()
    self.aboutdialog.hide()


if __name__ == "__main__":
  main = Buglump()
  gtk.main()
