#!/usr/bin/env python
# -*- coding: utf-8 -*-
version = '0.7'
"""

Copyright John Thornton 2012
"""
import os, sys

if sys.platform.startswith('linux'):
  current_path = os.path.dirname(os.path.realpath(__file__)) + '/'
  sys.path.append(current_path + '/data')

elif sys.platform.startswith('win'):
  print os.getcwd()

try:
  import pygtk
  pygtk.require('2.0')
except:
  pass
try:
  import gtk
except:
  print('GTK not available')
  sys.exit(1)


import database as db
import math
from decimal import *

class sfc:

  def __init__(self):

    self.builder = gtk.Builder()
    self.builder.add_from_file(current_path + "data/sfc.glade")
    self.builder.connect_signals(self)
    self.aboutdialog = self.builder.get_object("aboutdialog")
    self.aboutdialog.set_version(version)

    self.db = db.database()
    self.thread_form = []
    self.thread_size = []
    self.thread_class = []
    self.units = ''
    self.precision = 4

    self.formlist = gtk.ListStore(str)
    self.formlist.append(["Select a Form"])
    self.itemlist = self.db.thread_forms()
    for i in self.itemlist:
      self.formlist.append([i["form"]])
    self.form_combo = self.builder.get_object("thread_form_combo")
    self.form_combo.set_model(self.formlist)
    self.form_cell = gtk.CellRendererText()
    self.form_combo.pack_start(self.form_cell)
    self.form_combo.add_attribute(self.form_cell, 'text', 0)
    self.form_combo.set_active(0)

    self.thread_size_list = gtk.ListStore(str)
    self.thread_size_combo = self.builder.get_object("thread_size_combo")
    self.thread_size_combo.set_model(self.thread_size_list)
    self.thread_size_cell = gtk.CellRendererText()
    self.thread_size_combo.pack_start(self.thread_size_cell)
    self.thread_size_combo.add_attribute(self.thread_size_cell, 'text', 0)

    self.thread_class_list = gtk.ListStore(str)
    self.thread_class_combo = self.builder.get_object("thread_class_combo")
    self.thread_class_combo.set_model(self.thread_class_list)
    self.thread_class_cell = gtk.CellRendererText()
    self.thread_class_combo.pack_start(self.thread_class_cell)
    self.thread_class_combo.add_attribute(self.thread_class_cell, 'text', 0)

    # Drilling Page
    self.dm_cb = self.builder.get_object('drill_material_cb')
    self.dm_list = gtk.ListStore(str, int)
    self.dm_list.append(['Select Material', 0])
    self.dm_list.append(['Aluminum', 250])
    self.dm_list.append(['Steel 1018', 90])
    self.dm_list.append(['Steel 4140', 80])
    self.dm_list.append(['Steel A36', 50])
    self.dm_list.append(['Stainless 300', 50])
    self.dm_cb.set_model(self.dm_list)
    self.dm_cell = gtk.CellRendererText()
    self.dm_cb.pack_start(self.dm_cell, True)
    self.dm_cb.add_attribute(self.dm_cell, 'text', 0)
    self.dm_cb.set_active(0)

    self.drill_sfm_label = self.builder.get_object('drill_sfm_label')

    self.drill_feed_list = gtk.ListStore(str, str, str)
    self.drill_feed_list.append(["Select a Size", 0, 0])
    self.drill_feed_items = self.db.drill_feed()
    for i in self.drill_feed_items:
      self.drill_feed_list.append([i[0], i[1], i[2]])
    self.df_cb = self.builder.get_object('drill_feed_cb')
    self.df_cb.set_model(self.drill_feed_list)
    self.df_cell = gtk.CellRendererText()
    self.df_cb.pack_start(self.df_cell)
    self.df_cb.add_attribute(self.df_cell, 'text', 0)
    self.df_cb.set_active(0)

    self.drill_feed_label = self.builder.get_object('drill_feed_label')

    self.rpm_label = self.builder.get_object('rpm_label')
    self.ipm_label = self.builder.get_object('ipm_label')
    self.drill_rpm_str = ''


    self.window = self.builder.get_object("window1")
    self.window.set_title("Machinist Calculator")
    self.window.show_all()

  def on_thread_form_combo_changed(self, widget, data=None):
    self.index = widget.get_active()
    if self.index > 0:
      self.model = widget.get_model()
      self.item = [self.model[self.index][0]]
      self.thread_form = self.item
      if self.item == ['ISO']:
        self.units = 'mm'
        self.precision = 3
      else:
        self.units = '"'
        self.precision = 4
      self.thread_size_list.clear()
      self.thread_class_list.clear()
      self.itemlist = self.db.thread_size(self.item)
      self.thread_size_list.append(['Select a Size'])
      for i in self.itemlist:
        self.thread_size_list.append([i["size"]])
      self.thread_size_combo.set_active(0)
      self.clear_thread_info()

  def on_thread_size_combo_changed(self, widget, data=None):
    self.index = widget.get_active()
    if self.index > 0:
      self.model = widget.get_model()
      self.item = [self.model[self.index][0]]
      self.thread_size = self.item
      self.thread_class_list.clear()
      self.thread_class_list.append(['Select a Class'])
      self.itemlist = self.db.thread_class(self.item)
      for i in self.itemlist:
        self.thread_class_list.append([i["class"]])
      self.thread_class_combo.set_active(0)
      self.clear_thread_info()

  def on_thread_class_combo_changed(self, widget, data=None):
    self.index = widget.get_active()
    if self.index > 0:
      self.model = widget.get_model()
      self.item = [self.model[self.index][0]]
      self.thread_class = self.item
      self.thread_info = self.db.thread_info(self.thread_form, self.thread_size, self.thread_class)
      for i in self.thread_info:
        self.pitch = i['pitch']
        self.major_dia = i['major_dia']
        self.max_major_dia = i['max_major_dia']
        self.min_major_dia = i['min_major_dia']
        self.mid_major_dia = round((self.max_major_dia + self.min_major_dia)/2,self.precision)
        self.pitch_dia = i['pitch_dia']
        self.max_pitch_dia = i['max_pitch_dia']
        self.min_pitch_dia = i['min_pitch_dia']
        self.mid_pitch_dia = round((self.max_pitch_dia + self.min_pitch_dia)/2,self.precision)
        self.max_minor_dia = i['max_minor_dia']
        self.wire_size = i['wire_size']
      self.builder.get_object("major_dia").set_text(str(self.major_dia) + self.units)
      self.builder.get_object("max_major_dia").set_text(str(self.max_major_dia) + self.units)
      self.builder.get_object("mid_major_dia").set_text(str(self.mid_major_dia) + self.units)
      self.builder.get_object("min_major_dia").set_text(str(self.min_major_dia) + self.units)

      self.builder.get_object("pitch_dia").set_text(str(self.pitch_dia) + self.units)
      self.builder.get_object("max_pitch_dia").set_text(str(self.max_pitch_dia) + self.units)
      self.builder.get_object("mid_pitch_dia").set_text(str(self.mid_pitch_dia) + self.units)
      self.builder.get_object("min_pitch_dia").set_text(str(self.min_pitch_dia) + self.units)
      self.builder.get_object("max_minor_dia").set_text(str(self.max_minor_dia) + self.units)
      self.builder.get_object("sharp_v_depth").set_text(str(self.major_dia-self.pitch_dia) + self.units)
      if self.wire_size > 0:
        self.wire_const = (3 * self.wire_size)-(0.866025 * self.pitch)
        self.builder.get_object("thread_wire_dia").set_text(str(self.wire_size))
        self.builder.get_object("max_thread_wire").set_text(str(0.90 * self.pitch))
        self.builder.get_object("flynn_thread_wire").set_text(str(self.wire_size))
        self.builder.get_object("min_thread_wire").set_text(str(0.53 * self.pitch))
        self.builder.get_object("measure_over_wires").set_text(str(round(self.pitch_dia + self.wire_const,self.precision)))
        self.builder.get_object("measured_pitch_dia").set_text(self.calc_pitch_dia())
      else:
        self.builder.get_object("thread_wire_dia").set_text('n/a')
      self.drill_list()

  def drill_list(self):
    self.index = self.builder.get_object('thread_class_combo').get_active()
    if self.index > 0:
      self.fractional_drills = self.builder.get_object("cb_fractional_drills")
      self.number_drills = self.builder.get_object("cb_number_drills")
      self.letter_drills = self.builder.get_object("cb_letter_drills")
      self.metric_drills = self.builder.get_object("cb_metric_drills")
      self.range = []
      if self.fractional_drills.get_active():
        self.range.append("F")
      if self.number_drills.get_active():
        self.range.append("N")
      if self.letter_drills.get_active():
        self.range.append("L")
      if self.metric_drills.get_active():
        self.range.append("M")
      self.query = 'SELECT * FROM drills WHERE type IN (%s)' % ','.join('?' for i in self.range)
      self.query += " AND diam >= ? AND diam <= ?"
      if self.units == 'mm':
        self.drill100 = str(round(0.03937 * (self.major_dia  - (1.08254 * 1 * self.pitch)),self.precision))
        self.drill50 = str(round(0.03937 * (self.major_dia - (1.08254 * .5 * self.pitch)),self.precision))
      else:
        self.drill100 = str(round(self.major_dia  - (1.08254 * 1 * self.pitch),self.precision))
        self.drill50 = str(round(self.major_dia  - (1.08254 * .5 * self.pitch),self.precision))
      self.range.append(self.drill100)
      self.range.append(self.drill50)
      dlist = self.db.drill_list(self.query, self.range)
      self.drilllist = ''
      for i in dlist:
        self.drilllist += str(i["type"]) + '\n'
      self.builder.get_object("drill_type").set_text(self.drilllist)
      self.drilllist = ''
      for i in dlist:
        self.drilllist += str(i["size"]) + '\n'
      self.builder.get_object("drill_size").set_text(self.drilllist)
      self.drilllist = ''
      for i in dlist:
        self.drilllist += str(i["diam"]) + '\n'
      self.builder.get_object("drill_diameter").set_text(self.drilllist)
      self.drilllist = ''
      for i in dlist:
        if self.units == 'mm':
          self.percent = str(round((self.major_dia - (i["diam"] * 25.4)) / (0.01299 * self.pitch) ,2))
        else:
          self.percent = str(round((self.major_dia - i["diam"])/(0.1299 * self.pitch),2)*10)
        self.drilllist += self.percent + '%\n'
      self.builder.get_object("thread_percent").set_text(self.drilllist)

  def on_cb_fractional_drills_toggled(self, widget, data=None):
    self.drill_list()

  def on_cb_number_drills_toggled(self, widget, data=None):
    self.drill_list()

  def on_cb_letter_drills_toggled(self, widget, data=None):
    self.drill_list()

  def on_cb_metric_drills_toggled(self, widget, data=None):
    self.drill_list()

  def on_tip_radius_activate(self, widget, data=None):
    self.index = self.builder.get_object('thread_class_combo').get_active()
    if self.index > 0:
      self.sharp_v_depth = self.major_dia-self.pitch_dia
      self.builder.get_object("radius_depth").set_text(self.radius_depth(self.sharp_v_depth) + self.units)

  def radius_depth(self, sharp_v_depth):
    self.measure = float(self.builder.get_object('tip_radius').get_text())
    return str(round(sharp_v_depth-((self.measure * math.sqrt(3))-self.measure),self.precision))

  def on_measure_over_wires_activate(self, widget, data=None):
    print 'measure'
    self.builder.get_object("measured_pitch_dia").set_text(self.calc_pitch_dia())

  def on_thread_wire_dia_activate(self, widget, data=None):
    print 'wire'
    self.builder.get_object("measured_pitch_dia").set_text(self.calc_pitch_dia())

  def calc_pitch_dia(self):
    self.measure = float(self.builder.get_object('measure_over_wires').get_text())
    self.thread_wire_dia = float(self.builder.get_object('thread_wire_dia').get_text())
    self.const = (3 * self.thread_wire_dia)-(0.866025 * self.pitch)
    return str(round(self.measure-self.const,self.precision))

  def clear_thread_info(self):
    self.builder.get_object('major_dia').set_text('')
    self.builder.get_object('max_major_dia').set_text('')
    self.builder.get_object('mid_major_dia').set_text('')
    self.builder.get_object('min_major_dia').set_text('')
    self.builder.get_object('pitch_dia').set_text('')
    self.builder.get_object('max_pitch_dia').set_text('')
    self.builder.get_object('mid_pitch_dia').set_text('')
    self.builder.get_object('min_pitch_dia').set_text('')
    self.builder.get_object('max_minor_dia').set_text('')
    self.builder.get_object("sharp_v_depth").set_text('')
    self.builder.get_object("radius_depth").set_text('')
    self.builder.get_object("thread_wire_dia").set_text('')
    self.builder.get_object("measure_over_wires").set_text('')
    self.builder.get_object("measured_pitch_dia").set_text('')
    self.builder.get_object("drill_type").set_text('')
    self.builder.get_object("drill_size").set_text('')
    self.builder.get_object("drill_diameter").set_text('')
    self.builder.get_object("thread_percent").set_text('')

  def on_drill_material_changed(self, widget, data=None):
    self.index = widget.get_active()
    if self.index > 0:
      self.model = widget.get_model()
      self.item = [self.model[self.index][1]]
      self.drill_sfm_label.set_text('SFM ' + str(self.item[0]))
      self.drill_rpm_str = str(self.item[0])

  def on_drill_feed_cb_changed(self, widget, data=None):
    self.index = widget.get_active()
    if self.index > 0:
      self.model = widget.get_model()
      self.fpr = [self.model[self.index][1]]
      self.diameter = [self.model[self.index][2]]

      self.drill_feed_label.set_text('Feed/Rev ' + str(self.fpr[0]))
      self.sfm = Decimal(self.drill_rpm_str)
      self.rpm = (self.sfm * Decimal('12')) / (Decimal(self.diameter[0]) * Decimal(str(math.pi)))
      self.rpm_label.set_text(str(self.rpm).split('.')[0])

      self.ipm = Decimal(self.fpr[0]) * self.rpm
      self.ipm_label.set_text(str(self.ipm.quantize(Decimal('0.1'))))

  def on_window1_destroy(self, object, data=None):
    print "quit with cancel"
    gtk.main_quit()

  def on_file_quit_activate(self, menuitem, data=None):
    print "quit from menu"
    gtk.main_quit()

  def on_help_about_activate(self, menuitem, data=None):
    self.response = self.aboutdialog.run()
    self.aboutdialog.hide()

if __name__ == "__main__":
  main = sfc()
  gtk.main()
